# chemfp includes a vendered version of tqdm so the core functionality
# can be used without an external dependency.

# Some files which chemfp doesn't need have been removed to reduce the
# distribution size.


LICENSE = """
`tqdm` is a product of collaborative work.
Unless otherwise stated, all authors (see commit logs) retain copyright
for their respective work, and release the work under the MIT licence
(text below).

Exceptions or notable authors are listed below
in reverse chronological order:

* files: *
  MPLv2.0 2015-2021 (c) Casper da Costa-Luis
  [casperdcl](https://github.com/casperdcl).
* files: tqdm/_tqdm.py
  MIT 2016 (c) [PR #96] on behalf of Google Inc.
* files: tqdm/_tqdm.py setup.py README.rst MANIFEST.in .gitignore
  MIT 2013 (c) Noam Yorav-Raphael, original author.

[PR #96]: https://github.com/tqdm/tqdm/pull/96


Mozilla Public Licence (MPL) v. 2.0 - Exhibit A
-----------------------------------------------

This Source Code Form is subject to the terms of the
Mozilla Public License, v. 2.0.
If a copy of the MPL was not distributed with this file,
You can obtain one at https://mozilla.org/MPL/2.0/.


MIT License (MIT)
-----------------

Copyright (c) 2013 noamraph

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from ._monitor import TMonitor, TqdmSynchronisationWarning
from .std import (
    TqdmDeprecationWarning, TqdmExperimentalWarning, TqdmKeyError, TqdmMonitorWarning,
    TqdmTypeError, TqdmWarning, tqdm, trange)

__version__ = "4.62.2"

__all__ = ['tqdm', 'trange', 
           'TMonitor',
           'TqdmTypeError', 'TqdmKeyError',
           'TqdmWarning', 'TqdmDeprecationWarning',
           'TqdmExperimentalWarning',
           'TqdmMonitorWarning', 'TqdmSynchronisationWarning',
           '__version__']

# The following snippet is modified from
#   https://stackoverflow.com/questions/15411967/how-can-i-check-if-code-is-executed-in-the-ipython-notebook/39662359#39662359
def _is_in_notebook():
    try:
        get_ipython
    except NameError:
        # Not in a notebook
        return False
    
    shell = get_ipython().__class__.__name__
    if shell == "ZMQInteractiveShell": # Jupyter notebook or qtconsole
        return True
    elif shell == "TerminalInteractiveShell":
        return False  # Terminal running IPython
    else:
        return False  # Other type (?)
    
# tqdm keeps track of all tqdm instances, to allow multiple displays
# at the same time. It's therefore best to use the system tqdm, if
# available, otherwise fall back to the vendored version.

def disabled_tqdm(*args, **kwargs):
    from .std import tqdm
    kwargs["disable"] = True
    return tqdm(*args, **kwargs)

# Allow CHEMFP_PROGRESSBAR to disable the default progress bar.
import os
if os.environ.get("CHEMFP_PROGRESSBAR", "1") == "0":
    default_progressbar = disabled_tqdm
else:
    default_progressbar = None

def get_tqdm_constructor(progress=True):
    if callable(progress):
        return progress

    # Use explicit tests to True/False to help detect if the wrong
    # type was passed in.
    if progress == False:
        return disabled_tqdm
    
    elif progress == True:
        if default_progressbar is not None:
            return default_progressbar
        
        if _is_in_notebook():
            try:
                from tqdm.notebook import tqdm, IProgress
            except ImportError:
                from .notebook import tqdm, IProgress

            # Can only use the notebook version if ipywidgets is installed.
            if IProgress is not None:
                return tqdm
            # else fall back to the text-to-stderr version.

        try:
            from tqdm import tqdm
        except ImportError:
            from .std import tqdm

        return tqdm

    raise ValueError("progress must be a boolean or callable")
