# Copyright (c) 2010-2021 Andrew Dalke Scientific, AB (Sweden)
# All Rights Reserved.

## LICENSE ##

# This file contains proprietary and unpublished source code.
#
# You may use and modify this software ONLY under the terms of the
# License agreement you or your organization made with Andrew Dalke
# Scientific AB.
#
# If no other agreeement exists then you may use chemfp under the
# terms of the Chemfp Base License Agreement v1.0 or later.
# See https://chemfp.com/licenses/ for details.
#
# Unless required by applicable law or agreed to in writing, this
# software is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
# CONDITIONS OF ANY KIND, either express or implied.  See the License
# agreement for the specific language governing permissions and
# limitations under the License.

## /LICENSE ##

from .. import registry

def _check_toolkit_or_exit(progname, toolkit_name, toolkit_import):
    try:
        __import__(toolkit_import)
    except ImportError as err:
        raise SystemExit("Cannot run %s: It appears that %s is not installed: %s" % (
            progname, toolkit_name, err))

def run_ob2fps():
    _check_toolkit_or_exit("ob2fps", "Open Babel", "openbabel")
    from .ob2fps import run
    run()

ob2fps_command = registry.CommandConfig.from_sys_argv(
    "Generate Open Babel fingerprints", "fp_generation", run_ob2fps)
    
def run_oe2fps():
    _check_toolkit_or_exit("oe2fps", "OEChem", "openeye.oechem")
    from ..openeye_toolkit import is_licensed
    if not is_licensed():
        raise SystemExit("Cannot run oe2fps: OEChem is available but the license is not valid.")
    from openeye.oegraphsim import OEGraphSimIsLicensed
    if not OEGraphSimIsLicensed():
        raise SystemExit("Cannot run oe2fps: OEChem is available and licensed but the OEGraphSim license is not valid.")
        
    from .oe2fps import run
    run()

oe2fps_command = registry.CommandConfig.from_sys_argv(
    "Generate OpenEye fingerprints", "fp_generation", run_oe2fps)
    
def run_rdkit2fps():
    _check_toolkit_or_exit("rdkit2fps", "RDKit", "rdkit.Chem")

    from .rdkit2fps import run
    run()

rdkit2fps_command = registry.CommandConfig.from_sys_argv(
    "Generate RDKit fingerprints", "fp_generation", run_rdkit2fps)
    
def run_cdk2fps():
    import sys
    if sys.version_info.major == 2:
        raise SystemExit("Cannot run cdk2fps: Unable to use cdk2fps on Python 2")
    try:
        import jpype
    except ImportError as err:
        raise SystemExit("Cannot run cdk2fps: Cannot import jpype, which is required for chemfp to access the CDK jar: %s" % (err,))
    try:
        _check_toolkit_or_exit("cdk2fps", "CDK", "chemfp.cdk_toolkit")
    except jpype.JVMNotFoundException as err:
        raise SystemExit("Cannot run cdk2fps: %s" % (err,))

    from .cdk2fps import run
    run()

cdk2fps_command = registry.CommandConfig.from_sys_argv(
    "Generate RDKit fingerprints", "fp_generation", run_cdk2fps)
