import argparse
import sys

from .. import registry
from .. import fpb_io

parser = argparse.ArgumentParser(
    description="Show the low-level contents of an FPB file")

parser.add_argument("--chunk-info", action="store_true",
                        help="only print chunk information")
parser.add_argument("--skip-hash-table", action="store_true",
                        help="do not show or validate the HASH chunk")
parser.add_argument("--skip-ids", action="store_true",
                        help="do not show or validate the FPID chunk")
parser.add_argument("--show-ids", action="store_true",
                        help="include ids in the output")
parser.add_argument("--show-id-indices", action="store_true",
                        help="include id indices in the output")
parser.add_argument("--skip-fps", action="store_true",
                        help="do not show or validate fingerprints")
parser.add_argument("--show-fps", action="store_true",
                        help="include fingerprints in the output")
parser.add_argument("filename", help="FPB filename")


def main(argv=None):
    args = parser.parse_args(args=argv)
    fpb_dump_command(parser, args)

def fpb_dump_command(parser, args):
    try:
        f = open(args.filename, "rb")
    except Exception as err:
        sys.stderr.write("Cannot open FPB file: %s\n" % (err,))
        raise SystemExit(1)

    opts = fpb_io.Options(
        skip_hash_table = args.skip_hash_table,
        skip_ids = args.skip_ids,
        show_ids = args.show_ids,
        show_id_indices = args.show_id_indices,
        skip_fps = args.skip_fps,
        show_fps = args.show_fps)
    try:
        if args.chunk_info:
            fpb_io.describe_chunks(f, opts)
        else:
            fpb_io.describe(f, opts)
    finally:
        f.close()


command_config = registry.CommandConfig.from_argparse(
    parser, "fp_tools", fpb_dump_command).update(hidden=True)

if __name__ == "__main__":
    command_config.main()
