from .. import registry

import argparse

parser = argparse.ArgumentParser(
    description = "Show underlying cheminformatics toolkit availability")

def toolkits_command(parser, args):
    
    def _software():
        return software.partition("/")[2]
    
    try:
        import openbabel
    except ImportError as err:
        print("openbabel - not installed: %s" % (err,))
    else:
        from ..openbabel_toolkit import software
        print("openbabel - version %s installed" % (_software(),))

    try:
        import rdkit
    except ImportError as err:
        print("rdkit - not installed: %s" % (err,))
    else:
        from ..rdkit_toolkit import software
        print("rdkit - version %s installed" % (_software(),))

    try:
        import openeye
    except ImportError as err:
        print("oechem - not installed: %s" % (err,))
    else:
        from ..openeye_toolkit import software
        
        from openeye import oechem, oegraphsim
        if not oechem.OEChemIsLicensed():
            print("oechem - version %s installed but OEChem is not licensed" % (_software(),))
        elif not oegraphsim.OEGraphSimIsLicensed():
            print("oechem - version %s installed and OEChem licensed but OEGraphSim is not licensed" % (_software(),))
        else:
            print("oechem - version %s installed and both OEChem and OEGraphSim are licensed" % (_software(),))

    try:
        import jpype
    except ImportError:
        print("cdk - not installed: required Python module 'jpype' cannot be imported")
    else:
        try:
            from ..cdk_toolkit import software
        except ImportError as err:
            print("cdk - not installed: %s" % (err,))
        else:
            print("cdk - version %s installed" % (_software(),))

command_config = registry.CommandConfig.from_argparse(parser, "config", toolkits_command)

            
if __name__ == "__main__":
    command_config.main()
