import pandas
import numpy as np

#default_row_columns = ["id", "score"]

def check_columns2(columns):
    if len(columns) != 2:
        raise ValueError("exactly two column labels must be specified")

def to_pandas_from_row(self, columns):
    """INTERNAL FUNCTION. Example docstring

    Return a pandas DataFrame with the target ids and scores
        
    The first column contains the ids, the second column contains
    the ids. The default columns headers are "target_id" and
    "score". Use `columns` to specify different headers.

    :param columns: column names for the returned DataFrame
    :type columns: a list of two strings
    :returns: a pandas DataFrame
    """
    check_columns2(columns)
    return to_dataframe_2(columns, self.get_ids(), self.get_scores())

def to_dataframe_2(columns, ids, scores):
    d = {
        columns[0]: np.array(ids, dtype="O"),
        columns[1]: np.array(scores, dtype="f8"),
        }
    
    return pandas.DataFrame(
        data = d,
        columns = columns,
        )
    

#default_table_columns = ["query_id", "target_id", "score"]

def check_columns_and_empty3(columns, empty):
    if len(columns) != 3:
        raise ValueError("exactly three column labels must be specified")
    
    if empty is None:
        return None, None
    
    try:
        empty_id, empty_score = empty
    except Exception:
        raise ValueError("`empty` must be None or a 2-ple with (target id, score)") from None

    return empty_id, empty_score

def to_pandas_from_table(self, columns, empty, row_iter=None):
    """INTERNAL FUNCTION. Example docstring

    Example docstring

    Return a pandas DataFrame with query_id, target_id and score columns

    Each query has zero or more hits. Each hit becomes a row in the
    output table, with the query id in the first column, the hit
    target id in the second, and the hit score in the third.

    If a query has no hits then by default a row is added with the
    query id, '*' as the target id, and None as the score (which
    pandas will treat as a NA value).

    Use `empty` to specify different behavior for queries with no
    hits.  If `empty` is None then no row is added to the table. If
    `empty` is a 2-element tuple the first element is used as the
    target id and the second is used as the score.

    Use the DataFrame's `groupby()` method to group results by query
    id, for example:

        >>> import chemfp
        >>> df = chemfp.simsearch(queries="queries.fps", targets="targets.fps",
        ...        k=10, threshold=0.4, progress=False).to_pandas()
        >>> df.groupby("query_id").describe()

    :param columns: column names for the returned DataFrame
    :type columns: a list of three strings
    :param empty: the target id and score used for queries with
        no hits, or None to not include a row for that case
    :type empty: a list of two strings, or None
    :returns: a pandas DataFrame

    """
    empty_id, empty_score = check_columns_and_empty3(columns, empty)
    query_ids = []
    target_ids = []
    scores = []
    if row_iter is None:
        row_iter = zip(self.query_ids, self)
    for query_id, hits in row_iter:
        n = len(hits)
        if not n:
            if empty is None:
                continue
            query_ids.append(query_id)
            target_ids.append(empty_id)
            scores.append(empty_score)
        else:
            query_ids.extend([query_id] * n)
            target_ids.extend(hits.get_ids())
            scores.extend(hits.get_scores())
    
    data = {
        columns[0]: np.array(query_ids, dtype="O"),
        columns[1]: np.array(target_ids, dtype="O"),
        columns[2]: np.array(scores, dtype="f8"),
        }
    return pandas.DataFrame(
        data = data,
        columns = columns,
        )
