class CommandConfig:
    version = 1
    name = None
    def __init__(self, summary, group_name = None, hidden = False):
        self.summary = summary
        self.group_name = group_name
        self.hidden = hidden

    def update(self, **kwargs):
        for name in ("hidden",):
            if name in kwargs:
                setattr(self, name, kwargs[name])
        return self

    def register(self, name):
        self.name = name

    def run_command(self, args, namespace):
        raise NotImplementedError

    @staticmethod
    def from_argparse(parser, group_name, command):
        return _ArgParseCommandConfig(
            name = parser.prog,
            summary = parser.description,
            parser = parser,
            group_name = group_name,
            command = command,
            )

    @staticmethod
    def from_sys_argv(summary, group_name, func):
        return _SysArgvCommandConfig(summary, group_name, func)

class _ArgParseCommandConfig(CommandConfig):
    def __init__(self, name, summary, parser, group_name, command):
        super(_ArgParseCommandConfig, self).__init__(summary, group_name)
        self.name = name
        self.parser = parser
        self.command = command

    def __repr__(self):
        return f"{self.__class__.__name__}({self.name!r}, {self.parser!r}, {self.command!r})"

    def run_command(self, args, namespace):
        self.parser.prog += " " + self.name
        namespace = self.parser.parse_args(args, namespace)
        self.command(self.parser, namespace)

    def main(self, args=None):
        namespace = self.parser.parse_args(args)
        self.command(self.parser, namespace)

class _SysArgvCommandConfig(CommandConfig):
    def __init__(self, summary, group_name, func):
        super(_SysArgvCommandConfig, self).__init__(summary, group_name)
        self.summary = summary
        self.func = func

    def run_command(self, args, namespace):
        import sys
        sys.argv[:] = ["chemfp " + self.name] + args
        self.func()


class CommandGroupConfig:
    version = 1
    def __init__(self, summary, rank=10000):
        self.summary = summary
        self.rank = rank

    def register(self, name):
        self.name = name
        
## class ToolkitConfig:
##     version = 1

## class FingerprintTypeConfig:
##     version = 1
    
fp_generation_group = CommandGroupConfig(
    "Fingerprint generation", 100)

fp_search_group = CommandGroupConfig(
    "Fingerprint search", 200)

fp_conversion_group = CommandGroupConfig(
    "Fingerprint format conversion", 300)

fp_tools_group = CommandGroupConfig(
    "Fingerprint file tools", 400)

fp_diversity_group = CommandGroupConfig(
    "Diversity selection", 500)

toolkit_group = CommandGroupConfig(
    "Commands that depend on a cheminformatics toolkit", 600)

sdf_tools_group = CommandGroupConfig(
    "SD file tools", 700)

smi_tools_group = CommandGroupConfig(
    "SMILES file tools", 800)

config_group = CommandGroupConfig(
    "Configuration", 900)

concepts_group = CommandGroupConfig(
    "Concepts", 1000)
